#!/bin/sh

. "$_EXEC/indexmeta.sh"
. "$_EXEC/widgets.sh"

list_item() {
  local meta type length width height tags comment name display link
  meta="${1}"; type="${meta%%	*}"; meta="${meta#*	}"

  if [ "$type" = dir ]; then
    name="${meta%%	*}";
    display="$(HTML "$name")"; link="$(URL "${PATH_INFO%/}/$name")"
    printf '[a .list .dir href="%s" %s]' "${link}?${w_refuri#*\?}" "$name"
    return 0
  fi

  length="${meta%%	*}"; meta="${meta#*	}"
  width="${meta%%	*}"; meta="${meta#*	}"
  height="${meta%%	*}"; meta="${meta#*	}"
  tags="${meta%%	*}"; meta="${meta#*	}"
  comment="${meta%%	*}"; meta="${meta#*	}"
  name="${meta%%	*}"; meta="${meta#*	}"

  if [ "$type" = metashort ]; then
    name="$(list_fullname "$(UNSTRING "${name%${CR}}")")"
  fi
  if [ -f "$_DATA/${PATH_INFO%/}/$name" ]; then
    link="$(URL "${PATH_INFO%/}/${name#./}")"
    name="$(HTML "${PATH_INFO%/}/${name#./}")"
    printf '[div .list .file
              [a href="%s" [img src="%s?a=thumbnail"]][label . %s]
              [span .time %i:%02imin] [span .dim %ix%i] %s
              [checkbox "select" "%s" id="select_%s"][label for="select_%s" +]
            ]' \
      "$link" "$link" "${name##/}" \
      "$((length / 60))" "$((length % 60))" \
      "$width" "$height" \
      "$(printf '%s\n' "${tags#tags=}" \
         | sed -r "$UNSTRING"' s;^;,;; s;,+;,;g; s;,$;;;
                   :X s;,-?([^,]+)(,|$); [span .tag\n \1]\2;; tX;'
      )" "$name" "$link" "$link"
  else
    printf 'Canning record for nonexist file: %s\n' "$name" >&2
    meta_purge "$_DATA/$ITEM/$name"
  fi
}


[ "$FILTER" ] && list_fex="$(
  fex='p'
  STRING "$FILTER^" \
  | sed -E 's;\^;\n;g; s;[]\/\(\)\\\^\$\?\.\+\*\;\[\{\}];\\&;g' \
  | while read -r f; do
    [ "${f#\~}" ] || continue
    [ "${f#\~}" = "$f" ] \
    && fex="/(\ttags=([^\t]*,)?)(${f})((,[^\t]*)?\t)/{${fex}}" \
    || fex="/(\ttags=([^\t]*,)?)(${f#\~})((,[^\t]*)?\t)/d; ${fex}"
    printf '%s\n' "${fex}"
  done \
  | tail -n1
)"

list_fullname(){
  sn="$1"
  [ ! "${sn%%*/*}" ] && base="${sn%/*}" || base=.
  file="$(printf '%s' "$_DATA/$ITEM/$sn".*)"
  file="${file##*/}"
  [ -e "$_DATA/$ITEM/$base/${file}" ] \
  && printf '%s\n' "${base}/${file}"
}

list_filter(){
  if [ "$FILTER" ]; then
    printf "FEX: %s\n" "$list_fex" >&2
    sed -nE "$list_fex"
  elif [ "${SEARCH#!}" != "${SEARCH}" ]; then
    grep -aviEe "$(STRING "${SEARCH}" \
                 | sed -E ':x s;((^|[^\\])(\\\\)*)\+;\1 ;g; tx;
                            s;((^|[^\\])(\\\\)*)\\\+;\1+;g;
                            s; ;\\+;g;')"
  elif [ "${SEARCH}" ]; then
    grep -aiEe "$(STRING "${SEARCH}" \
                 | sed -E ':x s;((^|[^\\])(\\\\)*)\+;\1 ;g; tx;
                            s;((^|[^\\])(\\\\)*)\\\+;\1+;g;
                            s; ;\\+;g;')"
  else
    cat
  fi
}

list_order(){
  local fm fn fn al length ln h w t c name buffer l

  if [ $ORDER = Name ]; then
    sort -k6 |sed 's;^;metashort\t;;'
  elif [ $ORDER = Group ]; then
    sed -E '
      :X
      s;^([^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^ 0-9]*)-([0-9a-zA-Z_-]{11}\r|ph[0-9a-f]{13}\r|[0-9]{8}\r)(.*)$;\1-\r\3 \2\n;;
      t;
      s;^([^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^ 0-9]*)([0-9]+)(.*)$;\1\r\3 \2;;
      tX;' \
    | { sort -n -k7 -k8 -k9 -k10 -k11 |sort -s -k6,6 ; echo '0 0 0 tags= comment= _'; } \
    | while read -r length h w t c name; do
      if [ "${ln%% *}" = "${name%% *}" ]; then
        al=$((al + length))
        buffer="${buffer}${BR}$length	$h	$w	$t	$c	$name"
      else
	printf '%s\n' "$buffer" |while read -r l; do
          [ "$l" ] && printf '%s	%s\n' "$al" "$l"
        done
        al="$length"
        buffer="$length	$h	$w	$t	$c	$name"
      fi
      ln="$name"
    done \
    | sort -s -n -k1,1 |sed -E 's;^[0-9]+\t;metashort\t;;' \
    | sed -E ':X s;^([^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^\t]+\t[^\r]*)\r([^ ]*) ([^ ]+)( .*)?$;\1\3\2\4;; tX'
  elif [ $ORDER = Length ]; then
    sort -sn -k1 |sed 's;^;metashort\t;;'
  elif [ $ORDER = Date ]; then
    while read -r fm; do
      sn="${fm##*	}"
      fn="$(list_fullname "$(UNSTRING "${sn%${CR}}")")"
      printf '%i	%s	%s\n' \
             "$(stat -c %Y "$fn")" "${fm%	*}" "$fn"
    done \
    | sort -srn -k1 |sed -E 's;^[0-9]+\t;metalong\t;;'
  fi
}

list_filemeta(){
  local meta base cbase fm cachename
  base="$1"
  meta="$_DATA/$ITEM/$base/.index/meta"
  meta_dir "$_DATA/$ITEM/$base"

  cachename="$(printf '%s\n' "$mode" "$FILTER" "$SEARCH" "$ORDER" |sha1sum)"
  cachename="$_DATA/$ITEM/.index/${cachename%  -}.cache"

  if [ "$cachename" -nt "$meta" ] 2>&-; then
    cat "$cachename"
  else
    cbase="$(STRING "$base")"
    grep -axE '[0-9]+	[0-9]+	[0-9]+	tags=[^	]*	comment=[^	]*	.+' "$meta" \
    | while read -r fm; do
      printf '%s	%s/%s\n' "${fm%	*}" "$cbase" "${fm##*	}"
    done \
    | list_filter \
    | list_order \
    | { [ -d "${meta%meta}" ] && tee "$cachename" || cat; }
  fi
}

list_items() {
  local mode meta
  mode="$(COOKIE mode |grep -m1 -axE 'index|browse' || printf index )"
  
  if [ "$mode" = browse ]; then
    [ "$ITEM" ] && printf 'dir\t..\n'
    (cd "$_DATA/$ITEM";
      find ./ -type d \! -name .index -mindepth 1 -maxdepth 1 \
    ) | cut -d/ -f2- | sort |sed 's;^;dir\t;;'
    list_filemeta .
  elif [ "$mode" = index ]; then
    (cd "$_DATA/$ITEM";
      find ./ -path '*/.index/meta'
    ) | while read -r meta; do
      list_filemeta "${meta%/.index/meta}"
    done
  fi
}

list_paginate() {
  local page i c n end qry
  page="$(GET p |grep -axE '[0-9]+' || printf 1)"; c=1
  end=$((page + LISTSIZE))
  qry="${w_refuri#*\?}"; qry="${qry#&#112;&#61;*&#38;}"

  printf '[div .itemlist '
  while read -r i; do
    c=$((c + 1))
    [ $c -gt $page -a $c -le $end  ] && list_item "$i"
  done
  printf ']'

  [ $(( c % LISTSIZE )) -gt 0 ] \
  && end=$((c / LISTSIZE + 1)) \
  || end=$((c / LISTSIZE))

  printf '[div .pagination'
  for n in $( seq 1 $end ); do
    c=$(( (n - 1) * LISTSIZE + 1 ))
    [ $c = $page ] \
    && printf '[a .page .current href="%s" %s]' "?p=${c}&${qry}" "$n" \
    || printf '[a .page href="%s" %s]' "?p=${c}&${qry}" "$n"
  done
  printf ']'
}

printf 'Content-Type: text/html;charset=utf-8\r\n\r\n'

{ printf '
[!DOCTYPE HTML]
[html [head [title '
  w_bmname
  printf ' by %s]' "$ORDER"
  printf '
  [meta name="viewport" content="width=device-width"]
  [link rel=stylesheet href="/style.css" ]
] [body
  [div #navigation
    [a #t_bookmarks href="#bookmarks" &#x2605;]'
    w_search
    printf '
    [a #t_avsearch href="#advsearch" Advanced]
    [a #t_prefs href="#prefs" &#x2699;]
  ]'
  w_bookmarks
  w_advsearch
  w_prefs
  printf '
  [form method=POST action="?a=multitag"'
    list_items \
    | list_paginate
    [ -d "$_DATA/$ITEM/.index" ] && { printf '
    [div #editing'
      w_tagging
    printf '
    ]'; }
  printf '
  ]'
  [ ! -d "$_DATA/$ITEM/.index" ] && { printf '
  [div #editing'
    w_index
  printf '
  ]'; }
  printf '
] ]
'; } | "$_EXEC/cgilite/html-sh.sed"
